goog.require 'app.editor.Block'
goog.require 'app.BlockType'
goog.require 'app.EditorStore'
goog.require 'tests.Factory'

suite 'app.EditorStore', ->

  createBlock = tests.Factory.Block.create
  createStructure = tests.Factory.StructureBlock.create
  createDocument = tests.Factory.Document.createWithOpts

  suite 'dependencies', ->

    setup ->
      [@store, mocks] = tests.Factory.EditorStore.createWithMockedParams()
      @docStoreMock = mocks.docStore
      @transformMock = mocks.transform

      @store._notify = ->

    teardown ->
      @docStoreMock.verify()
      @docStoreMock.restore()
      @transformMock.verify()
      @transformMock.restore()

    test 'init ok', ->
      assert.ok @store

    test 'changeType', ->
      block = createBlock({text: 'Colosseum is...\nCircus Maximus'})
      structure = createBlock {blocks: [block]}
      @transformMock.expects('transformTo').once()
      @docStoreMock.expects('findBlockParent').returns(structure)
      @store.changeType(block, app.BlockType.List)

    test 'onSectionChange', ->
      nested = [{}, {}]
      block = {title: 'La Lavre', blocks: nested}
      @store.changeSection(block, {title: 'La La'})
      assert.equal block.title, 'La La'
      assert.deepEqual block.blocks, nested

  suite 'proxy methods to document store', ->
    setup ->
      [@store, mocks] = tests.Factory.EditorStore.createWithMockedParams()
      @store._notify = ->
      @docStoreMock = mocks.docStore

    teardown ->
      @docStoreMock.verify()
      @docStoreMock.restore()

    test 'appendNewBlock calls createBlock', ->
      block = createStructure()
      @docStoreMock.expects('createBlock').once().returns({title: 'new block'})
      @store.appendTextBlock(block)
      assert.equal block.blocks[0].title, 'new block'

    suite 'CRUD', ->
      setup ->
        @opera = createBlock {title: 'The Op éra Garnier'}
        @cinema = createBlock {title: 'Le Grand Rex', text: 'Paris largest cinema with 2,800 seats.'}
        @parisCulture = createBlock {
          title: 'Paris culture'
          type: app.BlockType.Section
          blocks: [
            @opera
            @cinema
          ]
        }
        @document = createDocument {
          title: 'Document'
          blocks: [
            createBlock {title: 'Rome culture'}
            @parisCulture
          ]
        }
        @store.document = @document

      test 'replaceBlock [mock]', ->
        newBlock = createBlock {title: 'The Louvre'}
        @docStoreMock.expects('findBlockParent').returns(@parisCulture)
        @store.replaceBlock(@cinema, newBlock)
        assert.equal @parisCulture.blocks[1], newBlock

      test 'deleteBlock', ->
        @docStoreMock.expects('findBlockParent').returns(@parisCulture)
        @store.deleteBlock(@cinema)
        assert.lengthOf @parisCulture.blocks, 1


    suite 'moving and activating blocks', ->
      setup ->
        [@store, mocks] = tests.Factory.EditorStore.createWithMockedParams()
        @store.docStore = tests.Factory.DocumentStore.createAndMockStorage()[0]

        @SPA = createStructure {
          type: app.BlockType.Section
          title: 'SPA'
          _id: 'SPA_ID'
          blocks: [{title: 'AngularJS', _id: 'AngularJS_ID'}, {title: 'Reactive', _id: 'Reactive_ID'}]
        }
        @LIBRARIES = createStructure {
          type: app.BlockType.Section
          title: 'Used libraries'
          blocks: [{title: 'React'}, {title: 'PouchDB'}]
        }
        @INTRODUCTION = createBlock {
          type: app.BlockType.Paragraph
          title: 'Introduction'
        }
        @document = createDocument {
          title: 'Use of JS in SPA'
          blocks: [@INTRODUCTION, @LIBRARIES, @SPA]
        }
        @store.docStore.document = @document

      suite 'add', ->

        test 'addBlockAfter', ->
          assert.lengthOf @document.blocks, 3
          @store.addBlockAfter(@LIBRARIES, app.BlockType.Image)
          assert.lengthOf @document.blocks, 4
          newBlock = @document.blocks[2]
          assert.equal newBlock.type, app.BlockType.Image

      suite 'move UP', ->
        test 'in the middle', ->
          assert.equal @SPA.blocks[0].title, 'AngularJS'
          assert.equal @SPA.blocks[1].title, 'Reactive'
          @store.moveUP(@SPA.blocks[1])
          assert.equal @SPA.blocks[0].title, 'Reactive'
          assert.equal @SPA.blocks[1].title, 'AngularJS', 'moved'

        test 'at TOP', ->
          assert.equal @SPA.blocks[0].title, 'AngularJS'
          @store.moveUP(@SPA.blocks[0])
          assert.equal @SPA.blocks[0].title, 'AngularJS', 'stayed'


      suite 'move DOWN', ->
        test 'at BOTTOM', ->
          assert.equal @SPA.blocks[1].title, 'Reactive'
          @store.moveDOWN(@SPA.blocks[1])
          assert.equal @SPA.blocks[1].title, 'Reactive', 'stayed'

        test 'at middle', ->
          assert.equal @LIBRARIES.blocks[0].title, 'React'
          assert.equal @LIBRARIES.blocks[1].title, 'PouchDB'
          @store.moveDOWN(@LIBRARIES.blocks[0])
          assert.equal @LIBRARIES.blocks[0].title, 'PouchDB'
          assert.equal @LIBRARIES.blocks[1].title, 'React'

      suite 'move LEFT', ->
        test 'move left', ->
          assert.equal @LIBRARIES.blocks[0].title, 'React'
          assert.equal @document.blocks.length, 3
          parentIndex = @document.blocks.indexOf(@LIBRARIES)

          @store.moveLEFT(@LIBRARIES.blocks[0])

          assert.equal @document.blocks.length, 4
          assert.equal @document.blocks[parentIndex].title, 'React', 'replace parent position'

      suite 'move RIGHT', ->
        test 'should move', ->
          assert.equal @document.blocks.length, 3
          assert.equal @document.blocks[0], @INTRODUCTION
          assert.equal @LIBRARIES.blocks.length, 2
          @store.moveRIGHT(@INTRODUCTION)
          assert.equal @document.blocks.length, 2
          assert.equal @LIBRARIES.blocks.length, 3
          assert.equal @LIBRARIES.blocks[0], @INTRODUCTION


      suite 'activating', ->
        test 'up', ->
          @store.activateUpTo(@SPA.blocks[1])
          assert.equal @store.appState.shouldActivateBlockId, 'AngularJS_ID'

        test 'down', ->
          @store.activateDownTo(@SPA.blocks[0])
          assert.equal @store.appState.shouldActivateBlockId, 'Reactive_ID'

        test 'inside', ->
          @store.activateInsideTo(@SPA)
          assert.equal @store.appState.shouldActivateBlockId, 'AngularJS_ID'

        test 'outside', ->
          @store.activateOutsideTo(@SPA.blocks[0])
          assert.equal @store.appState.shouldActivateBlockId, 'SPA_ID'
